<?php return array (
  'root' => 
  array (
    'pretty_version' => 'dev-master',
    'version' => 'dev-master',
    'aliases' => 
    array (
    ),
    'reference' => 'b5e6b4b4eb62e09232ee56e208eba9ae337d3ee8',
    'name' => 'wpserveur/wps-hide-login',
  ),
  'versions' => 
  array (
    'wpserveur/wps-hide-login' => 
    array (
      'pretty_version' => 'dev-master',
      'version' => 'dev-master',
      'aliases' => 
      array (
      ),
      'reference' => 'b5e6b4b4eb62e09232ee56e208eba9ae337d3ee8',
    ),
  ),
);
